@@ -22,34 +22,33 @@ package Sidef::Convert::Convert {
     }
 
     sub to_i {
-        Sidef::Types::Number::Number->new(Math::BigFloat->new($_[0]->get_value)->as_int);
+        Sidef::Types::Number::Number->new($_[0])->int;
     }
 
     *to_int = \&to_i;
 
     sub to_rat {
-        Sidef::Types::Number::Number->new(Math::BigRat->new($_[0]->get_value));
+        Sidef::Types::Number::Number->new($_[0]);
     }
 
     *to_rational = \&to_rat;
     *to_r        = \&to_rat;
 
     sub to_complex {
-        Sidef::Types::Number::Complex->new($_[0]->get_value);
+        Sidef::Types::Number::Complex->new($_[0]);
     }
 
     *to_c = \&to_complex;
 
     sub to_n {
-        Sidef::Types::Number::Number->new($_[0]->get_value);
+        Sidef::Types::Number::Number->new($_[0]);
     }
 
     *to_num    = \&to_n;
     *to_number = \&to_n;
 
     sub to_float {
-        my ($value) = $_[0]->get_value;
-        Sidef::Types::Number::Number->new(ref($value) eq 'Math::BigRat' ? $value->as_float : Math::BigFloat->new($value));
+        Sidef::Types::Number::Number->new($_[0])->float;
     }
 
     *to_f = \&to_float;
@@ -74,34 +73,6 @@ package Sidef::Convert::Convert {
 
     *to_re = \&to_regex;
 
-    sub to_byte {
-        Sidef::Types::Byte::Byte->new(CORE::ord($_[0]->get_value));
-    }
-
-    sub to_bytes {
-        Sidef::Types::Byte::Bytes->call($_[0]->get_value);
-    }
-
-    sub to_char {
-        Sidef::Types::Char::Char->call($_[0]->get_value);
-    }
-
-    sub to_chars {
-        Sidef::Types::Char::Chars->call($_[0]->get_value);
-    }
-
-    sub to_grapheme {
-        Sidef::Types::Grapheme::Grapheme->call($_[0]);
-    }
-
-    *to_graph = \&to_grapheme;
-
-    sub to_graphemes {
-        Sidef::Types::Grapheme::Graphemes->call($_[0]);
-    }
-
-    *to_graphs = \&to_graphemes;
-
     sub to_array {
         Sidef::Types::Array::Array->new($_[0]);
     }
