@@ -69,19 +69,31 @@ use $];
 
 HEADER
 
-        if (exists $opts{opt}{A}) {
-            my $accuracy = abs(int($opts{opt}{A}));
-            $opts{header} .= "local \$Math::BigFloat::accuracy = $accuracy;\n";
-        }
-
         if (exists $opts{opt}{P}) {
             my $precision = abs(int($opts{opt}{P}));
-            $opts{header} .= "local \$Math::BigFloat::precision = -$precision;\n";
+            $opts{header} .= "local \$Sidef::Types::Number::Number::PREC = $precision;\n";
         }
 
         if (exists $opts{opt}{M}) {
-            my $mode = lc($opts{opt}{M}) =~ s/\s+//rg;
-            $opts{header} .= "local \$Math::BigFloat::round_mode = '${mode}';\n";
+            my $round = unpack('A*', lc($opts{opt}{M}) =~ s/^\s+//r);
+
+            if ($round eq 'zero') {
+                $round = 1;
+            }
+            elsif ($round eq '+inf') {
+                $round = 2;
+            }
+            elsif ($round eq '-inf') {
+                $round = 3;
+            }
+            elsif ($round eq '+zero') {
+                $round = 4;
+            }
+            else {
+                $round = 0;
+            }
+
+            $opts{header} .= "local \$Sidef::Types::Number::Number::ROUND = $round;\n";
         }
 
         %addr    = ();
@@ -122,16 +134,19 @@ HEADER
         }
     }
 
-    sub _dump_reftype {
+    sub _get_reftype {
         my ($self, $obj) = @_;
 
         my $ref = ref($obj);
-        $self->_dump_string(
-                              $ref eq 'Sidef::Variable::ClassInit'     ? $self->_dump_class_name($obj)
-                            : $ref eq 'Sidef::Variable::Ref'           ? 'REF'
-                            : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Sidef::Types::Block::Block'
-                            :                                            $ref
-                           );
+            $ref eq 'Sidef::Variable::ClassInit'     ? $self->_dump_class_name($obj)
+          : $ref eq 'Sidef::Variable::Ref'           ? 'REF'
+          : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Sidef::Types::Block::Block'
+          :                                            $ref;
+    }
+
+    sub _dump_reftype {
+        my ($self, $obj) = @_;
+        $self->_dump_string($self->_get_reftype($obj));
     }
 
     sub _dump_string {
@@ -582,7 +597,10 @@ HEADER
             $code = 'local ' . $self->deparse_script($obj->{expr});
         }
         elsif ($ref eq 'Sidef::Variable::Global') {
-            $code = '$' . $obj->{class} . '::' . $obj->{name},;
+            $code = '$' . $obj->{class} . '::' . $obj->{name};
+        }
+        elsif ($ref eq 'Sidef::Variable::ClassVar') {
+            $code = '$' . $self->_get_reftype($obj->{class}) . '::' . $obj->{name};
         }
         elsif ($ref eq 'Sidef::Variable::Define') {
             my $name  = $obj->{name} . $refaddr;
@@ -603,13 +621,13 @@ HEADER
                 if (exists $self->{function} or exists $self->{class}) {
                     $self->top_add("use experimental 'lexical_subs';\n");
                     $code = "state sub $name() { state \$_$refaddr"
-                      . (defined($obj->{expr}) ? (" = " . $self->deparse_script($obj->{expr})) : '') . " }";
+                      . (defined($obj->{expr}) ? (" = do{" . $self->deparse_script($obj->{expr}) . '}') : '') . " }";
                 }
 
                 # Otherwise, use static constants
                 else {
                     $code = "sub $name() { state \$_$refaddr"
-                      . (defined($obj->{expr}) ? " = " . ($self->deparse_script($obj->{expr})) : '') . "}";
+                      . (defined($obj->{expr}) ? " = do{" . ($self->deparse_script($obj->{expr}) . '}') : '') . "}";
                 }
             }
             else {
@@ -621,7 +639,8 @@ HEADER
             my $value = "\$$name";
             if (not exists $obj->{inited}) {
                 $obj->{inited} = 1;
-                $code = "(state \$$name" . (defined($obj->{expr}) ? (" = " . $self->deparse_script($obj->{expr})) : '') . ")";
+                $code = "(state \$$name"
+                  . (defined($obj->{expr}) ? (' = do{' . $self->deparse_script($obj->{expr}) . '}') : '') . ")";
             }
             else {
                 $code = $value;
@@ -878,18 +897,16 @@ HEADER
             }
         }
         elsif ($ref eq 'Sidef::Types::Number::Number') {
-            my $value = $obj->get_value;
-
-            if (ref($value)) {
-                if (ref($value) eq 'Math::BigRat') {
-                    $value = (q{Math::BigRat->new('} . join('/', $value->parts) . q{')});
-                }
-                else {
-                    $value = (q{'} . $value->bstr . q{'});
-                }
-            }
-
-            $code = $self->make_constant($ref, 'new', "Number$refaddr", $value);
+            $code = $self->make_constant($ref, 'new', "Number$refaddr", "'" . $obj->_get_frac . "'");
+        }
+        elsif ($ref eq 'Sidef::Types::Number::Inf') {
+            $code = $self->make_constant($ref, 'new', "Inf$refaddr");
+        }
+        elsif ($ref eq 'Sidef::Types::Number::Ninf') {
+            $code = $self->make_constant($ref, 'new', "Ninf$refaddr");
+        }
+        elsif ($ref eq 'Sidef::Types::Number::Nan') {
+            $code = $self->make_constant($ref, 'new', "Nan$refaddr");
         }
         elsif ($ref eq 'Sidef::Types::String::String') {
             $code = $self->make_constant($ref, 'new', "String$refaddr", $self->_dump_string(${$obj}));
@@ -960,6 +977,14 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Block::Default') {
             $code = 'default' . $self->deparse_bare_block($obj->{block}->{code});
         }
+        elsif ($ref eq 'Sidef::Types::Block::With') {
+            my $dvar = $self->_dump_var($obj->{block}{init_vars}->{vars}[0]);
+            $code =
+                'do{ if (defined(my '
+              . $dvar . '='
+              . $self->deparse_args($obj->{expr}) . ')) '
+              . $self->deparse_bare_block($obj->{block}{code}) . '}';
+        }
         elsif ($ref eq 'Sidef::Types::Block::Gather') {
             $code =
                 "do {my \@_$refaddr;"
@@ -1054,7 +1079,11 @@ HEADER
             $code = $self->make_constant($ref, 'new', "Sig$refaddr");
         }
         elsif ($ref eq 'Sidef::Types::Number::Complex') {
-            $code = $self->make_constant($ref, 'new', "Complex$refaddr", "'" . ${$obj}->Re . "'", "'" . ${$obj}->Im . "'");
+
+            #$code = $self->make_constant($ref, 'new', "Complex$refaddr", "'" . ${$obj}->Re . "'", "'" . ${$obj}->Im . "'");
+            $code = $self->make_constant($ref, 'new', "Complex$refaddr",
+                                         "'" . $obj->re->get_value . "'",
+                                         "'" . $obj->im->get_value . "'");
         }
         elsif ($ref eq 'Sidef::Types::Array::Pair') {
             if (all { not defined($_) } @{$obj}) {
@@ -1141,24 +1170,6 @@ HEADER
               . $self->deparse_expr({self => $obj->{array}}) . '})'
               . $self->deparse_bare_block($obj->{block}->{code});
         }
-        elsif ($ref eq 'Sidef::Types::Byte::Bytes') {
-            $code = $self->_dump_array($ref, $obj);
-        }
-        elsif ($ref eq 'Sidef::Types::Byte::Byte') {
-            $code = $self->make_constant($ref, 'new', "Byte$refaddr", $obj->get_value);
-        }
-        elsif ($ref eq 'Sidef::Types::Char::Chars') {
-            $code = $self->_dump_array($ref, $obj);
-        }
-        elsif ($ref eq 'Sidef::Types::Char::Char') {
-            $code = $self->make_constant($ref, 'new', "Char$refaddr", $self->_dump_string(${$obj}));
-        }
-        elsif ($ref eq 'Sidef::Types::Grapheme::Graphemes') {
-            $code = $self->_dump_array($ref, $obj);
-        }
-        elsif ($ref eq 'Sidef::Types::Grapheme::Grapheme') {
-            $code = $self->make_constant($ref, 'new', "Grapheme$refaddr", $self->_dump_string(${$obj}));
-        }
         elsif ($ref eq 'Sidef::Types::Array::MultiArray') {
             $code = $self->make_constant($ref, 'new', "MultiArr$refaddr");
         }
