@@ -106,7 +106,7 @@ package Sidef::Deparse::Sidef {
 
         state $table = {
                         'inf'  => q{Number.inf},
-                        '-inf' => q{Number.inf('-')},
+                        '-inf' => q{Number.ninf},
                         'nan'  => q{Number.nan},
                         '0'    => q{Number},
                        };
@@ -205,6 +205,9 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Variable::Global') {
             $code = 'global ' . $obj->{class} . '::' . $obj->{name},;
         }
+        elsif ($ref eq 'Sidef::Variable::ClassVar') {
+            $code = $self->_dump_reftype($obj->{class}) . '!' . $obj->{name};
+        }
         elsif ($ref eq 'Sidef::Variable::Init') {
             $code = $self->_dump_init_vars($obj, 'var');
         }
@@ -218,7 +221,10 @@ package Sidef::Deparse::Sidef {
             my $name = $obj->{name};
             if (not exists $obj->{inited}) {
                 $obj->{inited} = 1;
-                $code = "define $name = (" . $self->deparse_script($obj->{expr}) . ')';
+                $code = "define $name";
+                if (defined $obj->{expr}) {
+                    $code .= ' = (' . $self->deparse_script($obj->{expr}) . ')';
+                }
             }
             else {
                 $code = $name;
@@ -228,7 +234,10 @@ package Sidef::Deparse::Sidef {
             my $name = $obj->{name};
             if (not exists $obj->{inited}) {
                 $obj->{inited} = 1;
-                $code = "const $name = (" . $self->deparse_script($obj->{expr}) . ')';
+                $code = "const $name";
+                if (defined $obj->{expr}) {
+                    $code .= ' = (' . $self->deparse_script($obj->{expr}) . ')';
+                }
             }
             else {
                 $code = $name;
@@ -238,7 +247,10 @@ package Sidef::Deparse::Sidef {
             my $name = $obj->{name};
             if (not exists $obj->{inited}) {
                 $obj->{inited} = 1;
-                $code = "static $name = (" . $self->deparse_script($obj->{expr}) . ')';
+                $code = "static $name";
+                if (defined $obj->{expr}) {
+                    $code .= ' = (' . $self->deparse_script($obj->{expr}) . ')';
+                }
             }
             else {
                 $code = $name;
@@ -360,7 +372,7 @@ package Sidef::Deparse::Sidef {
             $code = 'break';
         }
         elsif ($ref eq 'Sidef::Types::Block::Given') {
-            $code = 'given ' . $self->deparse_args($obj->{expr}) . $self->deparse_bare_block($obj->{block}{code});
+            $code = 'given ' . $self->deparse_args($obj->{expr}) . $self->deparse_expr({self => $obj->{block}});
         }
         elsif ($ref eq 'Sidef::Types::Block::When') {
             $code = 'when(' . $self->deparse_args($obj->{expr}) . ')' . $self->deparse_bare_block($obj->{block}{code});
@@ -371,6 +383,9 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Types::Block::Default') {
             $code = 'default' . $self->deparse_bare_block($obj->{block}->{code});
         }
+        elsif ($ref eq 'Sidef::Types::Block::With') {
+            $code = 'with ' . $self->deparse_args($obj->{expr}) . $self->deparse_expr({self => $obj->{block}});
+        }
         elsif ($ref eq 'Sidef::Types::Block::Next') {
             $code = 'next';
         }
@@ -468,20 +483,20 @@ package Sidef::Deparse::Sidef {
             $code = 'Sig';
         }
         elsif ($ref eq 'Sidef::Types::Number::Number') {
-            my $value = $obj->get_value;
-
-            if (ref($value)) {
-                if (ref($value) eq 'Math::BigRat') {
-                    $code = '(' . join('//', map { $self->_dump_number($_) } $value->parts()) . ')';
-                }
-                else {
-                    $code = $self->_dump_number($value->bstr);
-                }
-            }
-            else {
-                $code = $self->_dump_number($value);
+            $code = $self->_dump_number($obj->_get_frac);
+            if (index($code, '/') != -1) {
+                $code = qq{Number("$code")};
             }
         }
+        elsif ($ref eq 'Sidef::Types::Number::Inf') {
+            $code = 'Inf';
+        }
+        elsif ($ref eq 'Sidef::Types::Number::Ninf') {
+            $code = '-Inf';
+        }
+        elsif ($ref eq 'Sidef::Types::Number::Nan') {
+            $code = 'NaN';
+        }
         elsif ($ref eq 'Sidef::Types::Array::Array' or $ref eq 'Sidef::Types::Array::HCArray') {
             if (not @{$obj}) {
                 $code = 'Array';
@@ -526,41 +541,11 @@ package Sidef::Deparse::Sidef {
                     $code = 'Pair';
                 }
             }
-            elsif ($ref eq 'Sidef::Types::Byte::Bytes') {
-                if (not @{$obj}) {
-                    $code = 'Bytes';
-                }
-            }
-            elsif ($ref eq 'Sidef::Types::Byte::Byte') {
-                if (${$obj} == 0) {
-                    $code = 'Byte';
-                }
-            }
-            elsif ($ref eq 'Sidef::Types::Char::Chars') {
-                if (not @{$obj}) {
-                    $code = 'Chars';
-                }
-            }
-            elsif ($ref eq 'Sidef::Types::Grapheme::Grapheme') {
-                if (${$obj} eq "\0") {
-                    $code = 'Grapheme';
-                }
-            }
-            elsif ($ref eq 'Sidef::Types::Grapheme::Graphemes') {
-                if (not @{$obj}) {
-                    $code = 'Graphemes';
-                }
-            }
             elsif ($ref eq 'Sidef::Types::Glob::Dir') {
                 if (${$obj} eq '') {
                     $code = 'Dir';
                 }
             }
-            elsif ($ref eq 'Sidef::Types::Char::Char') {
-                if (${$obj} eq "\0") {
-                    $code = 'Char';
-                }
-            }
             elsif ($ref eq 'Sidef::Types::String::String') {
                 if (${$obj} eq '') {
                     $code = 'String';
