@@ -12,6 +12,8 @@ package Sidef::Types::Array::Array {
       q{0+}   => sub { scalar(@{$_[0]}) },
       q{bool} => sub { scalar(@{$_[0]}) };
 
+    require Sidef::Types::Number::Number;
+
     sub new {
         my (undef, @items) = @_;
         bless \@items, __PACKAGE__;
@@ -344,14 +346,14 @@ package Sidef::Types::Array::Array {
                 }
             }
 
-            return Sidef::Types::Number::Number->new($counter);
+            return Sidef::Types::Number::Number::_new_uint($counter);
         }
 
         foreach my $item (@{$self}) {
             $item eq $obj and $counter++;
         }
 
-        Sidef::Types::Number::Number->new($counter);
+        Sidef::Types::Number::Number::_new_uint($counter);
     }
 
     *count_by = \&count;
@@ -360,16 +362,17 @@ package Sidef::Types::Array::Array {
         my ($self, $array) = @_;
 
         if ($#{$self} != $#{$array}) {
-            return Sidef::Types::Bool::Bool->false;
+            return state $x = Sidef::Types::Bool::Bool->false;
         }
 
         foreach my $i (0 .. $#{$self}) {
             my ($x, $y) = ($self->[$i], $array->[$i]);
-            $x eq $y
-              or return Sidef::Types::Bool::Bool->false;
+            $x eq $y or do {
+                return state $x = Sidef::Types::Bool::Bool->false;
+            };
         }
 
-        return Sidef::Types::Bool::Bool->true;
+        return state $x = Sidef::Types::Bool::Bool->true;
     }
 
     sub ne {
@@ -718,7 +721,7 @@ package Sidef::Types::Array::Array {
         my ($self, $code) = @_;
 
         foreach my $i (0 .. $#{$self}) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i)))) {
+            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number::_new_uint($i)))) {
                 return $res;
             }
         }
@@ -732,7 +735,7 @@ package Sidef::Types::Array::Array {
         my ($self, $code) = @_;
 
         foreach my $i (0 .. $#{$self}) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i), $self->[$i]))) {
+            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]))) {
                 return $res;
             }
         }
@@ -752,7 +755,7 @@ package Sidef::Types::Array::Array {
 
         my @arr;
         foreach my $i (0 .. $#{$self}) {
-            push @arr, $code->run(Sidef::Types::Number::Number->new($i), $self->[$i]);
+            push @arr, $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]);
         }
 
         $self->new(@arr);
@@ -778,7 +781,7 @@ package Sidef::Types::Array::Array {
 
         my @arr;
         foreach my $i (0 .. $#{$self}) {
-            push(@arr, $self->[$i]) if $code->run(Sidef::Types::Number::Number->new($i), $self->[$i]);
+            push(@arr, $self->[$i]) if $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]);
         }
 
         $self->new(@arr);
@@ -793,8 +796,9 @@ package Sidef::Types::Array::Array {
         my $hash = Sidef::Types::Hash::Hash->new;
         foreach my $item (@{$self}) {
             my $key = $code->run(my $val = $item);
-            exists($hash->{$key}) || $hash->append($key, Sidef::Types::Array::Array->new);
-            $hash->{$key}->append($val);
+            my $str_key = "$key";
+            exists($hash->{$str_key}) or do { $hash->{$str_key} = Sidef::Types::Array::Array->new };
+            push @{$hash->{$str_key}}, $val;
         }
 
         $hash;
@@ -809,7 +813,7 @@ package Sidef::Types::Array::Array {
         }
 
         foreach my $key (keys %hash) {
-            $hash{$key} = Sidef::Types::Number::Number->new($hash{$key});
+            $hash{$key} = Sidef::Types::Number::Number::_new_uint($hash{$key});
         }
 
         Sidef::Types::Hash::Hash->new(%hash);
@@ -840,25 +844,25 @@ package Sidef::Types::Array::Array {
 
         foreach my $val (@{$self}) {
             if ($code->run($val)) {
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
             }
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     sub all {
         my ($self, $code) = @_;
 
-        @{$self} || return Sidef::Types::Bool::Bool->false;
+        @{$self} || return state $z = Sidef::Types::Bool::Bool->false;
 
         foreach my $val (@{$self}) {
             if (not $code->run($val)) {
-                return Sidef::Types::Bool::Bool->false;
+                return state $x = Sidef::Types::Bool::Bool->false;
             }
         }
 
-        Sidef::Types::Bool::Bool->true;
+        state $x = Sidef::Types::Bool::Bool->true;
     }
 
     sub assign_to {
@@ -883,20 +887,20 @@ package Sidef::Types::Array::Array {
             if (ref($obj) eq 'Sidef::Types::Block::Block') {
                 foreach my $i (0 .. $#{$self}) {
                     $obj->run($self->[$i])
-                      && return Sidef::Types::Number::Number->new($i);
+                      && return Sidef::Types::Number::Number::_new_uint($i);
                 }
-                return Sidef::Types::Number::Number->new(-1);
+                return Sidef::Types::Number::Number::_new_int(-1);
             }
 
             foreach my $i (0 .. $#{$self}) {
                 $self->[$i] eq $obj
-                  and return Sidef::Types::Number::Number->new($i);
+                  and return Sidef::Types::Number::Number::_new_uint($i);
             }
 
-            return Sidef::Types::Number::Number->new(-1);
+            return Sidef::Types::Number::Number::_new_int(-1);
         }
 
-        Sidef::Types::Number::Number->new(@{$self} ? 0 : -1);
+        Sidef::Types::Number::Number::_new_int(@{$self} ? 0 : -1);
     }
 
     *first_index = \&index;
@@ -908,22 +912,21 @@ package Sidef::Types::Array::Array {
             if (ref($obj) eq 'Sidef::Types::Block::Block') {
                 for (my $i = $#{$self} ; $i >= 0 ; $i--) {
                     $obj->run($self->[$i])
-                      && return Sidef::Types::Number::Number->new($i);
+                      && return Sidef::Types::Number::Number::_new_uint($i);
                 }
 
-                return Sidef::Types::Number::Number->new(-1);
+                return Sidef::Types::Number::Number::_new_int(-1);
             }
 
             for (my $i = $#{$self} ; $i >= 0 ; $i--) {
                 $self->[$i] eq $obj
-                  and return Sidef::Types::Number::Number->new($i);
+                  and return Sidef::Types::Number::Number::_new_uint($i);
             }
 
-            return Sidef::Types::Number::Number->new(-1);
-
+            return Sidef::Types::Number::Number::_new_int(-1);
         }
 
-        Sidef::Types::Number::Number->new($#{$self});
+        Sidef::Types::Number::Number::_new_int($#{$self});
     }
 
     *last_index = \&rindex;
@@ -994,7 +997,7 @@ package Sidef::Types::Array::Array {
 
     sub length {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new(scalar @{$self});
+        Sidef::Types::Number::Number::_new_uint(scalar @{$self});
     }
 
     *len  = \&length;    # alias
@@ -1002,7 +1005,7 @@ package Sidef::Types::Array::Array {
 
     sub end {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new($#{$self});
+        Sidef::Types::Number::Number::_new_int($#{$self});
     }
 
     *offset = \&end;
@@ -1039,7 +1042,7 @@ package Sidef::Types::Array::Array {
 
     sub pairs {
         my ($self) = @_;
-        __PACKAGE__->new(map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number->new($_), $self->[$_]) }
+        __PACKAGE__->new(map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number::_new_uint($_), $self->[$_]) }
                          0 .. $#{$self});
     }
 
@@ -1205,20 +1208,20 @@ package Sidef::Types::Array::Array {
         if (ref($obj) eq 'Sidef::Types::Block::Block') {
             foreach my $item (@{$self}) {
                 if ($obj->run($item)) {
-                    return Sidef::Types::Bool::Bool->true;
+                    return state $x = Sidef::Types::Bool::Bool->true;
                 }
             }
 
-            return Sidef::Types::Bool::Bool->false;
+            return state $x = Sidef::Types::Bool::Bool->false;
         }
 
         foreach my $item (@{$self}) {
             if ($item eq $obj) {
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
             }
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     *contain  = \&contains;
@@ -1232,31 +1235,35 @@ package Sidef::Types::Array::Array {
 
         foreach my $item (@{$self}) {
             if (ref($item) eq $ref || eval { $item->SUPER::isa($ref) }) {
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
             }
         }
 
-        return Sidef::Types::Bool::Bool->false;
+        return state $x = Sidef::Types::Bool::Bool->false;
     }
 
     sub contains_any {
         my ($self, $array) = @_;
 
         foreach my $item (@{$array}) {
-            return Sidef::Types::Bool::Bool->true if $self->contains($item);
+            if ($self->contains($item)) {
+                return state $x = Sidef::Types::Bool::Bool->true;
+            }
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     sub contains_all {
         my ($self, $array) = @_;
 
         foreach my $item (@{$array}) {
-            return Sidef::Types::Bool::Bool->false unless $self->contains($item);
+            unless ($self->contains($item)) {
+                return state $x = Sidef::Types::Bool::Bool->false;
+            }
         }
 
-        Sidef::Types::Bool::Bool->true;
+        state $x = Sidef::Types::Bool::Bool->true;
     }
 
     sub shift {
@@ -1377,9 +1384,9 @@ package Sidef::Types::Array::Array {
     sub cmp {
         my ($self, $arg) = @_;
 
-        state $mone = Sidef::Types::Number::Number->new(-1);
-        state $zero = Sidef::Types::Number::Number->new(0);
-        state $one  = Sidef::Types::Number::Number->new(1);
+        state $mone = Sidef::Types::Number::Number::_new_int(-1);
+        state $zero = Sidef::Types::Number::Number::_new_uint(0);
+        state $one  = Sidef::Types::Number::Number::_new_uint(1);
 
         my $l1 = $#{$self};
         my $l2 = $#{$arg};
@@ -1501,6 +1508,19 @@ package Sidef::Types::Array::Array {
         Sidef::Types::String::String->new(CORE::join($delim, @{$self}));
     }
 
+    sub join_bytes {
+        my ($self, $encoding) = @_;
+        state $x = require Encode;
+        $encoding = defined($encoding) ? $encoding->get_value : 'UTF-8';
+        Sidef::Types::String::String->new(
+            eval {
+                Encode::decode($encoding, CORE::join('', map { CORE::chr($_) } @{$self}));
+              } // return
+        );
+    }
+
+    *encode = \&join_bytes;
+
     sub reverse {
         my ($self) = @_;
         $self->new(CORE::reverse @{$self});
@@ -1517,9 +1537,10 @@ package Sidef::Types::Array::Array {
 
     sub copy {
         my ($self) = @_;
+        $self->new(@{$self});
 
-        state $x = require Storable;
-        Storable::dclone($self);
+        #state $x = require Storable;
+        #Storable::dclone($self);
     }
 
     sub delete_first {
@@ -1529,11 +1550,11 @@ package Sidef::Types::Array::Array {
             my $item = $self->[$i];
             if ($item eq $obj) {
                 CORE::splice(@{$self}, $i, 1);
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
             }
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     *remove_first = \&delete_first;
@@ -1545,11 +1566,11 @@ package Sidef::Types::Array::Array {
             my $item = $self->[$i];
             if ($item eq $obj) {
                 CORE::splice(@{$self}, $i, 1);
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
             }
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     *remove_last = \&delete_last;
@@ -1589,11 +1610,11 @@ package Sidef::Types::Array::Array {
             my $item = $self->[$i];
             $code->run($item) && do {
                 CORE::splice(@{$self}, $i, 1);
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
             };
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     *remove_first_if = \&delete_first_if;
@@ -1605,11 +1626,11 @@ package Sidef::Types::Array::Array {
             my $item = $self->[$i];
             $code->run($item) && do {
                 CORE::splice(@{$self}, $i, 1);
-                return Sidef::Types::Bool::Bool->true;
+                return state $x = Sidef::Types::Bool::Bool->true;
               }
         }
 
-        Sidef::Types::Bool::Bool->false;
+        state $x = Sidef::Types::Bool::Bool->false;
     }
 
     *remove_last_if = \&delete_last_if;
