@@ -5,18 +5,22 @@ package Sidef::Types::Range::RangeNumber {
       Sidef::Object::Object
       );
 
+    require Sidef::Types::Number::Number;
+
     use overload '@{}' => \&to_a;
 
     sub new {
         my (undef, $from, $to, $step) = @_;
 
         if (defined $to) {
-            $from = ref($from) ? $from->get_value : $from;
-            $to   = ref($to)   ? $to->get_value   : $to;
-            $step = ref($step) ? $step->get_value : defined($step) ? $step : 1;
+            Sidef::Types::Number::Number::_valid($from, $to, defined($step) ? $step : ());
+            $from = ref($from) ? $$from : $from;
+            $to   = ref($to)   ? $$to   : $to;
+            $step = ref($step) ? $$step : defined($step) ? $step : 1;
         }
         elsif (defined $from) {
-            $to   = ref($from) ? $from->get_value : $from;
+            Sidef::Types::Number::Number::_valid($from);
+            $to   = ref($from) ? $$from : $from;
             $from = 0;
             $step = 1;
         }
@@ -41,7 +45,8 @@ package Sidef::Types::Range::RangeNumber {
 
     sub by {
         my ($self, $step) = @_;
-        $self->{step} = $self->{step} < 0 ? -$step->get_value : $step->get_value;
+        Sidef::Types::Number::Number::_valid($step);
+        $self->{step} = $self->{step} < 0 ? -$$step : $$step;
         $self;
     }
 
@@ -77,8 +82,10 @@ package Sidef::Types::Range::RangeNumber {
     sub contains {
         my ($self, $num) = @_;
 
-        my $value = $num->get_value;
-        my ($min, $max) = map { $_->get_value } ($self->min, $self->max);
+        Sidef::Types::Number::Number::_valid($num);
+
+        my $value = $$num;
+        my ($min, $max) = ($self->{step} > 0 ? ($self->{from}, $self->{to}) : ($self->{to}, $self->{from}));
         my $step = $self->{step};
 
         Sidef::Types::Bool::Bool->new(
@@ -98,72 +105,35 @@ package Sidef::Types::Range::RangeNumber {
     sub each {
         my ($self, $code) = @_;
 
-        my $step  = $self->{step};
-        my $from  = $self->{from};
-        my $limit = $self->{to};
+        my $step = $self->{step};
+        my $from = $self->{from};
+        my $to   = $self->{to};
 
-        if ($step == 1 and $limit < (-1 >> 1) and $from < (-1 >> 1)) {
-
-            # Unpack limit
-            if (ref($from)) {
-                $self->{from} = $from = $from->numify;
-                $self->{step} = 1;
-            }
-            if (ref($limit)) {
-                $self->{step} = 1;
-                $self->{to} = $limit = $limit->numify;
-            }
-
-            foreach my $i ($from .. $limit) {
-                if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i)))) {
+        if (    $step == 1
+            and $to <   (-1 >> 1)
+            and $from < (-1 >> 1)
+            and (ref($from) ? Sidef::Types::Number::Number::_is_int($from) : 1)
+            and (ref($to)   ? Sidef::Types::Number::Number::_is_int($to)   : 1)) {
+            foreach my $i ($from .. $to) {
+                if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number::_new_int($i)))) {
                     return $res;
                 }
             }
         }
         elsif ($step > 0) {
-            if (ref($from) || ref($limit) || ref($step)) {
-
-                $from  = Math::BigFloat->new($from)  if not ref($from);
-                $limit = Math::BigFloat->new($limit) if not ref($limit);
-                $step  = Math::BigFloat->new($step)  if not ref($step);
-
-                for (my $i = $from->copy ; $i->bcmp($limit) <= 0 ; $i->badd($step)) {
-                    if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i->copy)))) {
-                        return $res;
-                    }
-                }
-            }
-            else {
-                for (my $i = $from ; $i <= $limit ; $i += $step) {
-                    if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i)))) {
-                        return $res;
-                    }
+            for (my $i = $from ; $i <= $to ; $i += $step) {
+                if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i)))) {
+                    return $res;
                 }
             }
         }
         else {
-            if (ref($from) || ref($limit) || ref($step)) {
-
-                $from  = Math::BigFloat->new($from)  if not ref($from);
-                $limit = Math::BigFloat->new($limit) if not ref($limit);
-                $step  = Math::BigFloat->new($step)  if not ref($step);
-
-                for (my $i = $from->copy ; $i->bcmp($limit) >= 0 ; $i->badd($step)) {
-                    if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i->copy)))) {
-                        return $res;
-                    }
-                }
-            }
-            else {
-                for (my $i = $from ; $i >= $limit ; $i += $step) {
-                    if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i)))) {
-                        return $res;
-                    }
+            for (my $i = $from ; $i >= $to ; $i += $step) {
+                if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($i)))) {
+                    return $res;
                 }
             }
         }
-
-        $self;
     }
 
     *for     = \&each;
@@ -172,35 +142,27 @@ package Sidef::Types::Range::RangeNumber {
     sub map {
         my ($self, $code) = @_;
 
-        my $step  = $self->{step};
-        my $from  = $self->{from};
-        my $limit = $self->{to};
+        my $step = $self->{step};
+        my $from = $self->{from};
+        my $to   = $self->{to};
 
         my @values;
-        if ($step == 1 and $limit < (-1 >> 1) and $from < (-1 >> 1)) {
-
-            # Unpack limit
-            if (ref($from)) {
-                $self->{from} = $from = $from->numify;
-                $self->{step} = 1;
-            }
-            if (ref($limit)) {
-                $self->{step} = 1;
-                $self->{to} = $limit = $limit->numify;
-            }
-
-            foreach my $i ($from .. $limit) {
-                push @values, $code->run(Sidef::Types::Number::Number->new($i));
+        if (    $step == 1
+            and $to <   (-1 >> 1)
+            and $from < (-1 >> 1)
+            and (ref($from) ? Sidef::Types::Number::Number::_is_int($from) : 1)
+            and (ref($to)   ? Sidef::Types::Number::Number::_is_int($to)   : 1)) {
+            foreach my $i ($from .. $to) {
+                push @values, $code->run(Sidef::Types::Number::Number::_new_int($i));
             }
         }
-
         elsif ($step > 0) {
-            for (my $i = $from ; $i <= $limit ; $i += $step) {
+            for (my $i = $from ; $i <= $to ; $i += $step) {
                 push @values, $code->run(Sidef::Types::Number::Number->new($i));
             }
         }
         else {
-            for (my $i = $from ; $i >= $limit ; $i += $step) {
+            for (my $i = $from ; $i >= $to ; $i += $step) {
                 push @values, $code->run(Sidef::Types::Number::Number->new($i));
             }
         }
@@ -213,37 +175,29 @@ package Sidef::Types::Range::RangeNumber {
     sub grep {
         my ($self, $code) = @_;
 
-        my $step  = $self->{step};
-        my $from  = $self->{from};
-        my $limit = $self->{to};
+        my $step = $self->{step};
+        my $from = $self->{from};
+        my $to   = $self->{to};
 
         my @values;
-        if ($step == 1 and $limit < (-1 >> 1) and $from < (-1 >> 1)) {
-
-            # Unpack limit
-            if (ref($from)) {
-                $self->{from} = $from = $from->numify;
-                $self->{step} = 1;
-            }
-            if (ref($limit)) {
-                $self->{step} = 1;
-                $self->{to} = $limit = $limit->numify;
-            }
-
-            foreach my $i ($from .. $limit) {
-                my $num = Sidef::Types::Number::Number->new($i);
+        if (    $step == 1
+            and $to <   (-1 >> 1)
+            and $from < (-1 >> 1)
+            and (ref($from) ? Sidef::Types::Number::Number::_is_int($from) : 1)
+            and (ref($to)   ? Sidef::Types::Number::Number::_is_int($to)   : 1)) {
+            foreach my $i ($from .. $to) {
+                my $num = Sidef::Types::Number::Number::_new_int($i);
                 push(@values, $num) if $code->run($num);
             }
         }
-
         elsif ($step > 0) {
-            for (my $i = $from ; $i <= $limit ; $i += $step) {
+            for (my $i = $from ; $i <= $to ; $i += $step) {
                 my $num = Sidef::Types::Number::Number->new($i);
                 push(@values, $num) if $code->run($num);
             }
         }
         else {
-            for (my $i = $from ; $i >= $limit ; $i += $step) {
+            for (my $i = $from ; $i >= $to ; $i += $step) {
                 my $num = Sidef::Types::Number::Number->new($i);
                 push(@values, $num) if $code->run($num);
             }
@@ -255,6 +209,40 @@ package Sidef::Types::Range::RangeNumber {
     *filter = \&grep;
     *select = \&grep;
 
+    sub reduce {
+        my ($self, $code) = @_;
+
+        my $step = $self->{step};
+        my $from = $self->{from};
+        my $to   = $self->{to};
+
+        my $value = Sidef::Types::Number::Number->new($from);
+        if (    $step == 1
+            and $to <   (-1 >> 1)
+            and $from < (-1 >> 1)
+            and (ref($from) ? Sidef::Types::Number::Number::_is_int($from) : 1)
+            and (ref($to)   ? Sidef::Types::Number::Number::_is_int($to)   : 1)) {
+            foreach my $i ($from + 1 .. $to) {
+                my $num = Sidef::Types::Number::Number::_new_int($i);
+                $value = $code->run($value, $num);
+            }
+        }
+        elsif ($step > 0) {
+            for (my $i = $from + $step ; $i <= $to ; $i += $step) {
+                my $num = Sidef::Types::Number::Number->new($i);
+                $value = $code->run($value, $num);
+            }
+        }
+        else {
+            for (my $i = $from + $step ; $i >= $to ; $i += $step) {
+                my $num = Sidef::Types::Number::Number->new($i);
+                $value = $code->run($value, $num);
+            }
+        }
+
+        $value;
+    }
+
     our $AUTOLOAD;
     sub DESTROY { }
 
