@@ -347,28 +347,25 @@ package Sidef::Types::String::String {
         $self->new(crypt($$self, $salt->get_value));
     }
 
-    sub hex {
+    sub bin {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new(CORE::hex($$self));
+        Sidef::Types::Number::Number->new($$self, 2);
     }
 
     sub oct {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new(CORE::oct($$self));
-    }
-
-    sub bin {
-        my ($self) = @_;
-        my $value = $$self;
-        Sidef::Types::Number::Number->new(CORE::oct(index($value, '0b') == 0 ? $value : ('0b' . $value)));
+        Sidef::Types::Number::Number->new($$self, 8);
     }
 
     sub num {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new($$self);
+        Sidef::Types::Number::Number->new($$self, 10);
     }
 
-    *dec = \&num;
+    sub hex {
+        my ($self) = @_;
+        Sidef::Types::Number::Number->new($$self, 16);
+    }
 
     sub substr {
         my ($self, $offs, $len) = @_;
@@ -649,11 +646,12 @@ package Sidef::Types::String::String {
 
     sub bytes {
         my ($self) = @_;
-        Sidef::Types::Byte::Bytes->call($$self);
+        my $string = $$self;
+        state $x = require bytes;
+        Sidef::Types::Array::Array->new(map { Sidef::Types::Number::Number->new(CORE::ord(bytes::substr($string, $_, 1))) }
+                                        0 .. bytes::length($string) - 1);
     }
 
-    *to_bytes = \&bytes;
-
     sub each_byte {
         my ($self, $code) = @_;
 
@@ -661,7 +659,9 @@ package Sidef::Types::String::String {
 
         state $x = require bytes;
         foreach my $i (0 .. bytes::length($string) - 1) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Byte::Byte->new(CORE::ord bytes::substr($string, $i, 1))))) {
+            if (
+                defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new(CORE::ord bytes::substr($string, $i, 1))))
+              ) {
                 return $res;
             }
         }
@@ -671,16 +671,14 @@ package Sidef::Types::String::String {
 
     sub chars {
         my ($self) = @_;
-        Sidef::Types::Char::Chars->call($$self);
+        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(//, CORE::join('', $$self)));
     }
 
-    *to_chars = \&chars;
-
     sub each_char {
         my ($self, $code) = @_;
 
         foreach my $char (CORE::split(//, $$self)) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Char::Char->new($char)))) {
+            if (defined(my $res = $code->_run_code(__PACKAGE__->new($char)))) {
                 return $res;
             }
         }
@@ -692,19 +690,17 @@ package Sidef::Types::String::String {
 
     sub graphemes {
         my ($self) = @_;
-        Sidef::Types::Grapheme::Graphemes->call($$self);
+        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } map { /\X/g } $$self);
     }
 
-    *graphs       = \&graphemes;
-    *to_graphemes = \&graphemes;
-    *to_graphs    = \&graphemes;
+    *graphs = \&graphemes;
 
     sub each_grapheme {
         my ($self, $code) = @_;
 
         my $str = $$self;
         while ($str =~ /(\X)/g) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Grapheme::Grapheme->new($1)))) {
+            if (defined(my $res = $code->_run_code(__PACKAGE__->new($1)))) {
                 return $res;
             }
         }
@@ -966,7 +962,7 @@ package Sidef::Types::String::String {
 
         if (ref($arg) eq 'Sidef::Types::Regex::Regex') {
             my $regex = $arg->{regex};
-            $counter += CORE::length($1) while $s =~ /($regex)/g;
+            ++$counter while $s =~ /$regex/g;
             return Sidef::Types::Number::Number->new($counter);
         }
         elsif (ref($arg) eq 'Sidef::Types::Block::Block') {
@@ -1018,6 +1014,8 @@ package Sidef::Types::String::String {
         Sidef::Types::Bool::Bool->new(Scalar::Util::looks_like_number($$self));
     }
 
+    *is_numeric = \&looks_like_number;
+
     sub is_palindrome {
         my ($self) = @_;
         Sidef::Types::Bool::Bool->new(${$self} eq CORE::reverse(${$self}));
