@@ -51,6 +51,8 @@ my %ignored_modules = map { $_ => 1 } qw (
   Sidef::Optimizer
   Sidef::Deparse::Sidef
   Sidef::Deparse::Perl
+  Sidef::Types::Number::Inf
+  Sidef::Types::Number::Ninf
   );
 
 my $name = basename($dir);
@@ -124,6 +126,7 @@ sub process_file {
 
         next if $sub eq 'get_value';
         next if $sub =~ /^[(_]/;
+        next if $sub =~ /^([[:upper:]])/;
         next if exists $ignored_subs{$sub};
 
         my $code;
